/**
 * Copyright Copyright 2007-8 Simon Andrews
 *
 *    This file is part of StackMeasure.
 *
 *    StackMeasure is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    StackMeasure is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with StackMeasure; if not, write to the Free Software
 *    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
import java.awt.BorderLayout;
import java.util.Enumeration;
import java.util.Vector;

import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.event.TableColumnModelEvent;
import javax.swing.event.TableColumnModelListener;
import javax.swing.table.TableModel;

public class DistanceDisplay extends JPanel implements ListSelectionListener, TableColumnModelListener {

	private final JTable table;
	private final TableModel model;
	private Vector listeners = new Vector();


	public DistanceDisplay (final DistanceMeasureSet distanceSet) {
		
		setLayout(new BorderLayout());
		model = distanceSet.getTableModel();
		table = new JTable(model);
		table.setDefaultRenderer(Object.class, new ColorCellRenderer());
		add(new JScrollPane(table),BorderLayout.CENTER);
		table.setTableHeader(null);
		table.setAutoResizeMode(JTable.AUTO_RESIZE_OFF); // Needed to allow a horizontal scrollbar
		table.setCellSelectionEnabled(true);
		table.getSelectionModel().addListSelectionListener(this);
		table.getColumnModel().addColumnModelListener(this);

	}

	public void addDistanceSelectionListener (DistanceSelectionListener d) {
		if (! listeners.contains(d)) {
			listeners.add(d);
		}
	}
	
	public void removeDistanceSelectionListener (DistanceSelectionListener d) {
		if (listeners.contains(d)) {
			listeners.remove(d);
		}
	}

	public void valueChanged(ListSelectionEvent lse) {
		if (table.getSelectedColumnCount() == 1 && table.getSelectedRowCount() == 1) {
			int col = table.getSelectedColumn();
			int row = table.getSelectedRow();
			Object o = model.getValueAt(row, col);
			if (o == null || ! (o instanceof DistanceMeasure) ) {
				System.out.println("Can't select this distance");
				Enumeration e = listeners.elements();
				while (e.hasMoreElements()) {
					((DistanceSelectionListener)e.nextElement()).clearSelections();
				}				
			}
			else {
				DistanceMeasure d = (DistanceMeasure)o;
				System.out.println("Selecting Distance "+d);

				//TODO: Come back and fix image selection for distances
//				((StackWindow)distanceSet.getCollection().getImagePlus().getWindow()).showSlice(d.firstPixel().slice()+((d.secondPixel().slice()-d.firstPixel().slice())/2));


				Enumeration e = listeners.elements();
				while (e.hasMoreElements()) {
					((DistanceSelectionListener)e.nextElement()).distanceSelected(d);
				}
			}
		}
		else {
			System.out.println("Multiple cells selected");
			Enumeration e = listeners.elements();
			while (e.hasMoreElements()) {
				((DistanceSelectionListener)e.nextElement()).clearSelections();
			}
		}
	}

	public void columnAdded(TableColumnModelEvent e) {}

	public void columnMarginChanged(ChangeEvent e) {}

	public void columnMoved(TableColumnModelEvent e) {}

	public void columnRemoved(TableColumnModelEvent e) {}

	public void columnSelectionChanged(ListSelectionEvent e) {
		valueChanged(null);
	}
}
