/**
 * Copyright Copyright 2007-8 Simon Andrews
 *
 *    This file is part of StackMeasure.
 *
 *    StackMeasure is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    StackMeasure is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with StackMeasure; if not, write to the Free Software
 *    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
import java.awt.Color;


public class Pixel {

	private StackMeasureFile file;
	private int r;
	private int g;
	private int b;
	private int h;
	private int s;
	private int l;
	private int x;
	private int y;
	private int slice;
	
	public Pixel (StackMeasureFile file, byte r, byte g, byte b, int slice, int x, int y) {
		// This is used for RGB pixels
		this.file = file;
		this.r = r & 0xff;
		this.g = g & 0xff;
		this.b = b & 0xff;
		this.x = x;
		this.y = y;
		this.slice = slice;
		rgbToHsl();
	}
	
	public Pixel (StackMeasureFile file, int slice, int x, int y) {
		// This is used for black and white pixel data
		// where we don't have a colour
		this.x = x;
		this.y = y;
		this.slice = slice;
		this.file = file;
	}
	
	public StackMeasureFile file () {
		return file;
	}
	
	public byte r () {
		return (byte)r;
	}
	public byte g () {
		return (byte)g;
	}
	public byte b () {
		return (byte)b;
	}
	public int h () {
		// The hue is still used even for black and white pixels.  In these
		// cases we return the hue from the colour associated with the file.
		
		if (file.isRGB()) {
			return h;
		}
		else {
			int r = file.getColour().getRed();
			int g = file.getColour().getGreen();
			int b = file.getColour().getBlue();
			
			float [] hsb = Color.RGBtoHSB(r, g, b, null);
						
			return (int)(hsb[0]*360);
		}
	}
	public int s () {
		return s;
	}
	public int l () {
		return l;
	}
	public int x () {
		return x;
	}
	public int y () {
		return y;
	}
	public int slice () {
		return slice;
	}
	
	private void rgbToHsl() {
		float [] hsbvals = Color.RGBtoHSB(r, g, b, null);
		h = (int)(hsbvals[0]*360);
		s = (int)(hsbvals[1]*100);
		l = (int)(hsbvals[2]*100);
	}
	
	public static byte [] intToByteArray (int i) {
		// Because we know that this is always going to be RGB data
		// we don't have to worry about the length of i.  We just use
		// the last 3 bytes.
		byte [] b = new byte [3];
		for (int x=0;x<b.length;x++) {
			b[2-x] = (byte)(i>>>(x*8));
		}
						
		return b;
	}
}
