/**
 * Copyright Copyright 2007-8 Simon Andrews
 *
 *    This file is part of StackMeasure.
 *
 *    StackMeasure is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    StackMeasure is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with StackMeasure; if not, write to the Free Software
 *    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Graphics;

import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;

public class ProgressDialog extends JDialog implements Runnable, StackMeasureProcessorListener{

	private JLabel label;
	private int current = 0;
	private int total = 1;
	private ProgressBar progressBar = new ProgressBar();


	public ProgressDialog (JDialog parent) {
		super(parent,"Processing Data...");
		setSize(300,75);
		setLocationRelativeTo(parent);
		
		label = new JLabel("",JLabel.CENTER);
		getContentPane().setLayout(new BorderLayout());
		getContentPane().add(label,BorderLayout.CENTER);
		getContentPane().add(progressBar,BorderLayout.SOUTH);
		
		Thread t = new Thread(this);
		t.start();
		setDefaultCloseOperation(JDialog.DO_NOTHING_ON_CLOSE);
		setVisible(true);
	}
	
	public ProgressDialog (JFrame parent) {
		super(parent,"Processing Data...");
		setSize(300,75);
		setLocationRelativeTo(parent);
		
		label = new JLabel("",JLabel.CENTER);
		getContentPane().setLayout(new BorderLayout());
		getContentPane().add(label,BorderLayout.CENTER);
		getContentPane().add(progressBar,BorderLayout.SOUTH);
		
		Thread t = new Thread(this);
		t.start();
		setDefaultCloseOperation(JDialog.DO_NOTHING_ON_CLOSE);
		setVisible(true);
	}
	
	public void run() {
		try {
			Thread.sleep(500);
		} catch (InterruptedException e) {}
	}

	public void progressUpdate(int current, int total, String message) {
		this.current = current;
		this.total = total;
		progressBar.repaint();
		label.setText(message);		
	}

	public void exceptionReceived(Exception e) {
		JOptionPane.showMessageDialog(this,e.getMessage(),"Error",JOptionPane.ERROR_MESSAGE);
		e.printStackTrace();
		setVisible(false);
		dispose();
	}
	
	public void processingComplete() {
		setVisible(false);
		dispose();
	}
	
	private class ProgressBar extends JPanel {
		
		public void paint (Graphics g) {
			super.paint(g);
			g.setColor(Color.RED);
			g.fillRect(0,0,(int)(getWidth()*((float)current/total)),getHeight());
		}
		
	}
	
}
